// Fill out your copyright notice in the Description page of Project Settings.

#include "ViveportSDKPrivatePCH.h"
#include "ViveportSubscriptionDemo.h"


// Sets default values for this component's properties
UViveportSubscriptionDemo::UViveportSubscriptionDemo()
{
    // Set this component to be initialized when the game starts, and to be ticked every frame.  You can turn these features
    // off to improve performance if you don't need them.
    PrimaryComponentTick.bCanEverTick = true;
}


// Called when the game starts
void UViveportSubscriptionDemo::BeginPlay()
{
    Super::BeginPlay();
    mySubscriptionInitCallback.SetDemoPtr(this);
    myIsReadyCallback.SetDemoPtr(this);

    UViveportApi::Init(&mySubscriptionInitCallback, APP_ID);
}

void UViveportSubscriptionDemo::EndPlay(const EEndPlayReason::Type EndPlayReason)
{
    Super::EndPlay(EndPlayReason);

    // Call ViveportApi::Shutdown()
    UViveportApi::Shutdown(&mySubscriptionShutdownCallback);
}


// Called every frame
void UViveportSubscriptionDemo::TickComponent(
    float DeltaTime,
    ELevelTick TickType,
    FActorComponentTickFunction* ThisTickFunction
    )
{
    Super::TickComponent(DeltaTime, TickType, ThisTickFunction);
}

/***************************************************************
*					MySubscriptionInitCallback
***************************************************************/

void UViveportSubscriptionDemo::MySubscriptionInitCallback::OnSuccess()
{
    FString fstring("Init success.");
    GEngine->AddOnScreenDebugMessage(-1, 15.0f, FColor::White, fstring);

    UViveportSubscription::IsReady(mDemo->GetIsReadyCallback());
}

void UViveportSubscriptionDemo::MySubscriptionInitCallback::OnFailure(int error_code)
{
    FString fstring = FString::Printf(TEXT("Init failure. error=%d"), error_code);
    GEngine->AddOnScreenDebugMessage(-1, 15.0f, FColor::White, fstring);
}

/***************************************************************
*					MySubscriptionShutdownCallback
***************************************************************/

void UViveportSubscriptionDemo::MySubscriptionShutdownCallback::OnSuccess()
{
    UE_LOG(ViveportSDK, Log, TEXT("[UViveportSubscriptionDemo][MySubscriptionShutdownCallback] Shutdown success."));
    FString fstring("Shutdown success.");
    GEngine->AddOnScreenDebugMessage(-1, 15.0f, FColor::White, fstring);
}

void UViveportSubscriptionDemo::MySubscriptionShutdownCallback::OnFailure(int error_code)
{
    UE_LOG(ViveportSDK, Error, TEXT("[UViveportSubscriptionDemo][MySubscriptionShutdownCallback] Shutdown failure. Error = %d"), error_code);
    FString fstring = FString::Printf(TEXT("Shutdown failure. error=%d"), error_code);
    GEngine->AddOnScreenDebugMessage(-1, 15.0f, FColor::White, fstring);
}

/***************************************************************
*					MyIsReadyCallback
***************************************************************/

void UViveportSubscriptionDemo::MyIsReadyCallback::OnSuccess()
{
    UE_LOG(ViveportSDK, Log, TEXT("[UViveportSubscriptionDemo][MyIsReadyCallback] IsReady success."));
    FString fstring = FString::Printf(TEXT("Is ready success"));
    GEngine->AddOnScreenDebugMessage(-1, 15.0f, FColor::White, fstring);

    // Get IsSubscribed
    bool isViveportClientNeedToUpdate = false;
    bool isSubscribed = UViveportSubscription::IsSubscribed(isViveportClientNeedToUpdate);
    if (isViveportClientNeedToUpdate)
    {
        UE_LOG(ViveportSDK, Log, TEXT("[UViveportSubscriptionDemo][GetSubscriptionIsSubscribed] Please update VIVEPORT client."));
        fstring = FString::Printf(TEXT("Please update VIVEPORT client."));
        GEngine->AddOnScreenDebugMessage(-1, 15.0f, FColor::White, fstring);
    }
    else
    {
        if (isSubscribed)
        {
            UE_LOG(ViveportSDK, Log, TEXT("[UViveportSubscriptionDemo][GetSubscriptionIsSubscribed] User subscribed this content."));
            fstring = FString::Printf(TEXT("User subscribed this content"));
            GEngine->AddOnScreenDebugMessage(-1, 15.0f, FColor::White, fstring);
        }
        else
        {
            UE_LOG(ViveportSDK, Log, TEXT("[UViveportSubscriptionDemo][GetSubscriptionIsSubscribed] User didn't subscribe this content."));
            fstring = FString::Printf(TEXT("User didn't subscribe this content"));
            GEngine->AddOnScreenDebugMessage(-1, 15.0f, FColor::White, fstring);
        }
    }
}

void UViveportSubscriptionDemo::MyIsReadyCallback::OnFailure(int error_code)
{
    UE_LOG(ViveportSDK, Error, TEXT("[UViveportSubscriptionDemo][MyIsReadyCallback] IsReady failure. Code = %d"), error_code);
    FString fstring = FString::Printf(TEXT("IsReady failure. Code = %d"), error_code);
    GEngine->AddOnScreenDebugMessage(-1, 15.0f, FColor::White, fstring);
}
